/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.actions;

import org.eclipse.ajdt.ui.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;

import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.*;
import org.eclipse.jface.window.*;

import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;

import org.eclipse.ui.*;

import java.io.*;
import java.util.*;

/**
 * This action is triggered from a popup menu on a build configuration
 * file. It makes the target the current build config file for the project.
 */
public class BuildSelectorAction implements IWorkbenchWindowActionDelegate {

	private IWorkbenchWindow activeWindow = null;
	private IFile currentlySelectedBuildFile = null;

	/**
	 *  Executed when button clicked or popup menu "Select this build file" clicked
	 */
	public void run(IAction action) {

		AspectJPlugin ajp = AspectJPlugin.getDefault();
		final IProject buildFileProject = currentlySelectedBuildFile.getProject();
		AspectJPlugin.setBuildConfigurationFile(
			buildFileProject,
			currentlySelectedBuildFile);
		AspectJPlugin.getDefault().setCurrentProject( buildFileProject );
		// AMC addition
		Shell activeShell = AspectJPlugin.getDefault().getActiveWorkbenchWindow().getShell();
		IRunnableWithProgress op = new IRunnableWithProgress( ) {
			public void run( IProgressMonitor pm ) {
				try {
					buildFileProject.build(
						IncrementalProjectBuilder.FULL_BUILD, 
						AspectJPlugin.ID_BUILDER,
						null,
						pm );
				} catch ( CoreException cEx ) {
					AspectJPlugin.getDefault().getErrorHandler().handleError( 
						"Build on select error", cEx );	
				} catch ( Exception e ) {
					System.out.println( e );
					e.printStackTrace();
				}
			}
		};
		try { new ProgressMonitorDialog( activeShell).run( true, true, op ); }
		catch ( Exception e  ) {
			AspectJPlugin.getDefault( ).getErrorHandler().handleError(
			 "Auto build on select failed", e );
		}
		// end AMC addition

	}

	/**
	 * Selection has changed - if we've selected a build file then remember it
	 * as the new project build config.
	 */
	public void selectionChanged(IAction action, ISelection selection) {

		if (selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			Object first = structuredSelection.getFirstElement();
			if (first instanceof IFile) {
				currentlySelectedBuildFile = (IFile) first;
			}
		}
	}

	/**
	 * From IWorkbenchWindowActionDelegate
	 */
	public void dispose() {}

	/**
	 * From IWorkbenchWindowActionDelegate
	 */
	public void init(IWorkbenchWindow window) {
		activeWindow = window;
	}}