/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.editor;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.ui.texteditor.AbstractTextEditor;

import org.eclipse.ui.part.EditorPart;
import java.util.List;
import java.util.ListIterator;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberInputStream;
import java.io.LineNumberReader;

import org.aspectj.ajde.*;
import org.aspectj.ajde.ui.BuildConfigModel;
import org.aspectj.ajde.ui.BuildConfigNode;
import org.eclipse.core.resources.IMarker;
import org.eclipse.jdt.ui.*;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.resources.*;
import org.eclipse.ui.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ajdt.ui.*;
import org.eclipse.ajdt.internal.core.AJDTEventTrace;
import org.eclipse.ajdt.internal.core.AJDTUtils;
import org.eclipse.ajdt.internal.core.resources.AJDTIcon;
import org.eclipse.ajdt.internal.core.resources.AspectJImages;
import org.eclipse.ajdt.internal.ui.ajde.CompilerMonitor;
import org.eclipse.core.resources.*;
import java.util.Enumeration;
import java.util.Vector;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.*;
import org.eclipse.jface.viewers.*;
import java.util.Hashtable;
import java.util.Iterator;
import org.eclipse.core.resources.IResource;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.events.*;
import org.eclipse.core.resources.IProject;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.aspectj.asm.*;
import org.eclipse.core.runtime.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import java.io.StringWriter;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;


public class BuildConfigEditor
	extends EditorPart
	implements SelectionListener {

	/** is a save required? */
	boolean isDirty = false;

	/**  The tree representation on the AJDT side */
	private Tree tree = null; 

	/** The model tree returned from AJDE */
	BuildConfigModel model = null;

	/** The file we are "editing" */
	IFileEditorInput fileInput;

	/** Map from StructureNode to TreeItem used by gotoMarker */
	private Hashtable nodeMap;

	ImageDescriptor compilationUnitImgDesc;
	ImageDescriptor directoryImgDesc;
//	Image compilationUnitImage;
//	Image directoryImage;
	
	/**
	 * Double clicking on an lst file opens it, double clicking on anything
	 * else selects/checks it.
	 * @see SelectionListener#widgetDefaultSelected(SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e) {
		TreeItem ti = (TreeItem) e.item;
		BuildConfigNode node = (BuildConfigNode) ti.getData( );
		if ( node != null && 
			 node.getBuildConfigNodeKind() == BuildConfigNode.Kind.FILE_LST ) {
			 String relativePath = node.getResourcePath();
			 IPath path = fileInput.getFile().getProjectRelativePath();
			 path = path.removeLastSegments(1);
			 path = path.append( relativePath );
			 IFile file = fileInput.getFile().getProject().getFile( path );	
			 if ( file != null ) {
				 try {		 
					 AspectJPlugin.getDefault().
					 	getWorkbench().
					 	getActiveWorkbenchWindow().
					 	getActivePage().
			 			openEditor( file );
				 } catch ( Exception ex ) { System.err.println( ex ); }
			 }
		} else {
			widgetSelected( e );
		}
	}

	/**
	 * @see SelectionListener#widgetSelected(SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		if ((e.detail & SWT.CHECK) > 0) {
			TreeItem ti = (TreeItem) e.item;
			boolean checked = ti.getChecked();
			setTreeNodeStatus(ti, checked);
			ensureTreeCheckboxConsistency(tree);
		}				
	}
	
	
	/**
	 * @see IEditorPart#doSave(IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		Ajde.getDefault().getConfigurationManager().writeModel( model );
		clearDirty( );
		// after a save we must clear markers and then reload the model to
		// get an updated set of errors and warnings
		clearMarkers( fileInput );
		buildModel( fileInput );
		CompilerMonitor.showOutstandingProblems( );
				
		tree.removeAll();
		populateTree();		
	}

	/**
	 * @see IEditorPart#doSaveAs()
	 */
	public void doSaveAs() {
		// save as not supported
		doSave( null );
	}

	/**
	 * @see IEditorPart#gotoMarker(IMarker)
	 */
	public void gotoMarker(IMarker marker) {
		try {
			Integer lineNumber = (Integer) marker.getAttribute( IMarker.LINE_NUMBER );
			String sourceFile = AJDTUtils.getResourcePath( marker.getResource() );
			StructureNode node = model.findNodeForSourceLine( sourceFile, lineNumber.intValue() );
			if ( node != null ) {
				// now find the TreeItem for this node and select it
				TreeItem tItem = (TreeItem) nodeMap.get( node );
				if ( tItem != null ) {
					TreeItem[] selectionTarget = new TreeItem[] { tItem };
					tree.setSelection( selectionTarget );
				}
			}
		} catch ( CoreException cEx ) {
			AspectJPlugin.getDefault().getErrorHandler().handleError( "Unable to go to marker", cEx);
		}
	}

	/**
	 * @see IEditorPart#init(IEditorSite, IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
		throws PartInitException {

		setSite( site );
		setInput( input );
		setTitle( input.getName() );

		if (!(input instanceof IFileEditorInput)) {
			throw new PartInitException("InvalidInput");
		}
		
		fileInput = (IFileEditorInput) input;
		isDirty = false;

		clearMarkers( fileInput );
		buildModel( fileInput );
		CompilerMonitor.showOutstandingProblems( );
		
		AJDTEventTrace.editorOpened( fileInput.getFile() );
	}

	/**
	 * @see IEditorPart#isDirty()
	 */
	public boolean isDirty() {
		return isDirty;
	}

	/**
	 * @see IEditorPart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}

	/**
	 * @see IWorkbenchPart#createPartControl(Composite)
	 */
	public void createPartControl(Composite parent) {
		tree = new Tree( parent, SWT.CHECK | SWT.CASCADE );
		tree.addSelectionListener( this );
		populateTree( );
	}

	/**
	 * @see IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		tree.setFocus();
	}


	public void dispose( ) {
		AJDTEventTrace.editorClosed( fileInput.getFile() );
		super.dispose();	
	}
	
	/** clears any markers that exist for the given file */
	private void clearMarkers( IFileEditorInput input ) {
		try {
			((IResource) input.getFile()).deleteMarkers(
				IMarker.PROBLEM,
				true,
				IResource.DEPTH_INFINITE);

		} catch (CoreException e) {
			AspectJPlugin.getDefault().getErrorHandler().handleError( e.getMessage(), e);
		}
	}
	
	/** build the AJDE model for this config file */
	private void buildModel( IFileEditorInput input ) {
		IProject thisFilesProject = input.getFile().getProject();
		IPath workspacePath = thisFilesProject.getLocation();
		workspacePath = workspacePath.removeLastSegments(1);
		String fullPath = workspacePath.toOSString() + 
		                  input.getFile().getFullPath().toOSString();
		nodeMap = new Hashtable( );
		model =
			Ajde.getDefault().getConfigurationManager().buildModel( fullPath );
	}


	/**
	 * Prior to calling this method, the model should have
	 * been populated. Build a tree that represents it.
	 */
	private void populateTree( ) {
		if ( model != null ) {
			populateBranch( tree, (BuildConfigNode) model.getRoot() );
			ensureTreeCheckboxConsistency( tree );
		}
	}


	/**
	 * Populate a branch of the tree...
	 */	
	private void populateBranch(Widget parent, BuildConfigNode node) {
		TreeItem tItem;

		for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
			BuildConfigNode childNode = (BuildConfigNode) it.next();
			if (parent instanceof Tree) {
				tItem = new TreeItem(((Tree) parent), SWT.NULL);

			} else {
				tItem = new TreeItem(((TreeItem) parent), SWT.NULL);
			}
			
			tItem.setText(childNode.getName());

			tItem.setImage( createNodeImage( childNode ) );
			tItem.setData( childNode );
			tItem.setChecked(childNode.isActive());
			tItem.setExpanded(true);
			nodeMap.put( childNode, tItem );
			populateBranch(tItem, childNode);
		}
	}

	/**
	 * Ensure the correct nodes are open and check marks are in place
	 */
	private boolean ensureTreeCheckboxConsistency(Widget where) {
		TreeItem[] tis;
		boolean oneselected = false;

		if (where instanceof TreeItem) {
			TreeItem ti = (TreeItem) where;
			tis = ti.getItems();
			oneselected = false;
			boolean graychildren = false;

			int count = 0;
			for (int i = 0; i < tis.length; i++)
				if (ensureTreeCheckboxConsistency(tis[i])) {
					ti.setChecked(true);
					ti.setExpanded(true);
					oneselected = true;
					if (tis[i].getGrayed())
						graychildren = true;
					count++;
				}

			if (oneselected) {
				if (count != tis.length || graychildren)
					ti.setGrayed(true);
				else
					ti.setGrayed(false);
			}

			if (count == 0 && tis.length != 0) {
				ti.setChecked(false);
				ti.setGrayed(false);
				ti.setExpanded(false);
			}

			return oneselected || ti.getChecked();
		} else {
			tis = ((Tree) where).getItems();
			for (int i = 0; i < tis.length; i++)
				ensureTreeCheckboxConsistency(tis[i]);
			return false;
		}
	}

	/**
	 * Check or uncheck a given tree node, with cascade into child nodes.
	 * Build model is updated accordingly
	 */
	private void setTreeNodeStatus( TreeItem item, boolean include ) {
		BuildConfigNode node = (BuildConfigNode) item.getData();
		if ( include ) { includeNodeInConfig( node ); }
		else { excludeNodeFromConfig( node ); }		
		// recurse
		TreeItem[] children = item.getItems();
		for ( int i = 0; i < children.length; i++ ) {
			children[i].setChecked( include );
			setTreeNodeStatus( children[i], include );
		}	
	}


	/**
	 * Change event needed to force update of display to include 
	 * dirty marker in title
	 */
	private void setDirty( ) {
		isDirty = true;
		firePropertyChange(IWorkbenchPart.PROP_TITLE);
	}
	
	/**
	 * Change event needed to force update of display to exclude 
	 * dirty marker in title
	 */
	private void clearDirty( ) {
		isDirty = false;
		firePropertyChange(IWorkbenchPart.PROP_TITLE);
	}
	
	/**
	 * This node should be included in the config if it is not already
	 * present. Mark dirty if a change is required
	 */
	private void includeNodeInConfig( BuildConfigNode node ) {
		if ( !node.isActive() ) {
			node.setActive( true );
			setDirty( );
		}
	}
	
	/**
	 * This node should be excluded from the config if it is not already
	 * absent. Mark dirty if a change is required
	 */
	private void excludeNodeFromConfig( BuildConfigNode node ) {
		if ( node.isActive( ) ) {
			node.setActive( false );
			setDirty( );	
		}	
	}

	/**
	 * Create the image for a node, taking into account any
	 * possible error or warning overlays.
	 */		
	private Image createNodeImage( BuildConfigNode node ) {
		ImageDescriptor baseDescriptor;
		BuildConfigNode.Kind kind = node.getBuildConfigNodeKind();
		AJDTIcon icon = (AJDTIcon) AspectJImages.registry().getStructureIcon( kind );		
		baseDescriptor = icon.getImageDescriptor();
		// now for overlays...
		int overlayFlags = 0;
		if ( kind == BuildConfigNode.Kind.ERROR ) {
			overlayFlags = JavaElementImageDescriptor.ERROR;
		}
		
		StructureMessage sMessage = node.getMessage();
		if ( sMessage != null ) {
			if ( sMessage.getKind() == StructureMessage.Kind.ERROR ) {
				overlayFlags = JavaElementImageDescriptor.ERROR;
			} else if ( sMessage.getKind() == StructureMessage.Kind.WARNING ) {
				overlayFlags = JavaElementImageDescriptor.WARNING;	
			}
		}
				
				
		return AJDTUtils.decorate( baseDescriptor, overlayFlags).createImage();
	}
	

}
